local iGuild = LibStub("AceAddon-3.0"):GetAddon("iGuild");
local L = LibStub("AceLocale-3.0"):GetLocale("iGuild");

local function strsplit(delimiter, text)
  local list = {}
  local pos = 1
  if strfind("", delimiter, 1) then -- this would result in endless loops
    --error("delimiter matches empty string!")
  end
  while 1 do
    local first, last = strfind(text, delimiter, pos)
    if first then -- found?
      tinsert(list, strsub(text, pos, first-1))
      pos = last+1
    else
      tinsert(list, strsub(text, pos))
      break
    end
  end
  return list
end

local function strjoin(delimiter, list)
  local len = getn(list)
  if len == 0 then 
    return "" 
  end
  local string = list[1]
  for i = 2, len do 
    string = string .. delimiter .. list[i] 
  end
  return string
end

local function CreateConfig()
	local db = {
		type = "group",
		name = "iGuild",
		order = 1,
		get = function(info)
			return iGuild.db.Column[info.arg.k][info.arg.v];
		end,
		set = function(info, value)
			iGuild.db.Column[info.arg.k][info.arg.v] = value;
		end,
		args = {
			Infotext1 = {
				type = "description",
				name = ("|cffff00ff%s |cffffffff%s|r |cffff00ff%s|r"):format(L["Write the columns in the order you wish to display into the following textbox. Example:"],
					"level, name", L["will display Level and Name columns."]),
				fontSize = "medium",
				order = 1,
			},
			Display = {
				type = "input",
				name = "",
				order = 2,
				width = "full",
				validate = function(info, value)
					local list = strsplit(",%s*", value);
					for i = 1, #list do
						if( not iGuild.Columns[list[i]] ) then
							return L["Invalid column name!"];
						end
					end
					return true;
				end,
				get = function(info)
					return strjoin(", ", iGuild.db.Display);
				end,
				set = function(info, value)
					local list = strsplit(",%s*", value);
					iGuild.db.Display = list;
				end,
			},
			Infotext2 = {
				type = "description",
				fontSize = "medium",
				order = 3,
			},
			Sorting = {
				type = "select",
				name = L["Sorting"],
				order = 5,
				values = {
					["name"] = L["By Name"],
					["level"] = L["By Level"],
					["class"] = L["By Class"],
					["rank"] = L["By Guildrank"],
					["points"] = L["By Achievement Points"],
				},
				get = function() return iGuild.db.Sort end,
				set = function(info, value) iGuild.db.Sort = value end,
			},
		},
	};
	
	local colnames = {};
	local i = 6;
	for k, v in pairs(iGuild.Columns) do
		table.insert(colnames, k);
		
		db.args["Header"..i] = {
			type = "header",
			name = L["Column"]..": "..v.Label,
			order = i
		};
		i = i + 1;
		
		db.args["ShowLabel"..i] = {
			type = "toggle",
			name = L["Show Label"],
			order = i,
			arg = {
				k = k,
				v = "ShowLabel",
			},
		};
		i = i + 1;
		
		db.args["Align"..i] = {
			type = "select",
			name = L["Align"],
			order = i,
			values = {
				["LEFT"] = L["Left"],
				["CENTER"] = L["Center"],
				["RIGHT"] = L["Right"],
			},
			arg = {
				k = k,
				v = "Align",
			},
		};
		i = i + 1;
	end
	
	db.args.Infotext2.name = ("|cffff00ff%s: |cffffffff%s|r |cffff00ff%s|r\n"):format(L["Available columns"], strjoin(", ", colnames), L["- feel free to combine them like you want!"]);
	
	return db;
end

function iGuild:CreateDB()
	iGuild.CreateDB = nil;
	
	return { profile = {
		Display = { "level", "name", "acmpoints", "zone", "rank", "notecombi" },
		Sort = "name",
		Column = {
			level = {
				ShowLabel = false,
				Align = "RIGHT",
			},
			name = {
				ShowLabel = true,
				Align = "LEFT",
			},
			zone = {
				ShowLabel = true,
				Align = "CENTER",
			},
			rank = {
				ShowLabel = true,
				Align = "LEFT",
			},
			note = {
				ShowLabel = true,
				Align = "LEFT",
			},
			officernote = {
				ShowLabel = true,
				Align = "LEFT",
			},
			notecombi = {
				ShowLabel = true,
				Align = "LEFT",
			},
			acmpoints = {
				ShowLabel = true,
				Align = "CENTER",
			},
		},
	}};
end

function iGuild:OpenOptions()
	LibStub("AceConfigDialog-3.0"):Open("iGuild");
end

LibStub("AceConfig-3.0"):RegisterOptionsTable("iGuild", CreateConfig);
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("iGuild");
LibStub("AceConfigDialog-3.0"):SetDefaultSize("iGuild", 400, 500);
_G.SlashCmdList["IGUILD"] = iGuild.OpenOptions;
_G["SLASH_IGUILD1"] = "/iguild";