------------------------------------------
-- Setting up libraries
------------------------------------------

local AddonName = "iGuild"
iGuild = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceTimer-3.0");

local QTip = LibStub("LibQTip-1.0");
local Tourist = LibStub("LibTourist-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

local Tooltip;
local RosterTimer;

iGuild.Broker = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "Loading...",
	icon = "Interface\\Addons\\iGuild\\Fubar_GuildFu",

	OnClick = function(_, button)
		if( button == "LeftButton" ) then
			if( IsAltKeyDown() and CanGuildInvite() ) then
				if( StaticPopup_FindVisible("ADD_GUILDMEMBER") ) then
					StaticPopup_Hide("ADD_GUILDMEMBER");
				else
					StaticPopup_Show("ADD_GUILDMEMBER");
				end
			else
				ToggleGuildFrame(1);
			end
		elseif( button == "RightButton" ) then
			iGuild:OpenOptions();
		end
	end,

	OnEnter = function(anchor)
		if( not IsInGuild() ) then
			return;
		end
			
		Tooltip = QTip:Acquire(iGuild);
		Tooltip:SetAutoHideDelay(0.01, anchor);
		Tooltip:SmartAnchorTo(anchor);
		iGuild:UpdateTooltip();
		Tooltip:Show();
	end,
});

------------------------------------------
-- Variables and colors
------------------------------------------

local NULL = "";
local COLUMN_HEADER_ROW = 0;

local COLOR_GOLD = "|cfffed100";
local COLOR_OFFICER = "|cff40c040";

local ClassColors = {};
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end

------------------------------------------
-- Sorting and Columns
------------------------------------------

iGuild.Sort = {
	name = function(a, b)
		return a.charName < b.charName;
	end,
	level = function(a, b)
		if( a.charLevel < b.charLevel ) then
			return true;
		elseif( a.charLevel > b.charLevel ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end,
	class = function(a, b)
		if( a.charClass < b.charClass ) then
			return true;
		elseif( a.charClass > b.charClass ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end,
	rank = function(a, b)
		if( a.guildRank < b.guildRank ) then
			return true;
		elseif( a.guildRank > b.guildRank ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end,
	points = function(a, b)
		if( a.acmPoints > b.acmPoints ) then
			return true;
		elseif( a.acmPoints < b.acmPoints ) then
			return false;
		else
			return iGuild.Sort.name(a, b);
		end
	end
};

iGuild.Columns = {
	level = {
		Label = L["Level"],
		Brush = function(v)
			local c = GetQuestDifficultyColor(v.charLevel);
			return ("|cff%02x%02x%02x%s|r"):format(c.r *255, c.g *255, c.b *255, v.charLevel);
		end,
	},
	name = {
		Label = L["Name"],
		Brush = function(v)
			return ("%s%s%s|r"):format(ClassColors[v.charClass], v.charStatus or NULL, v.charName);
		end,
	},
	zone = {
		Label = L["Zone"],
		Brush = function(v)
			local r, g, b = Tourist:GetFactionColor(v.charZone);
			return ("|cff%02x%02x%02x%s|r"):format(r *255, g *255, b *255, v.charZone);
		end,
	},
	rank = {
		Label = L["Rank"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v.guildRank);
		end,
		Script = function(_, v, button)
			if( button == "LeftButton" and CanGuildPromote() ) then
				GuildPromote(v.charName);
			end
			if( button == "RightButton" and CanGuildDemote() ) then
				GuildDemote(v.charName);
			end
		end,
		ScriptUse = function() return ( CanGuildPromote() or CanGuildDemote() ) end,
	},
	note = {
		Label = L["Note"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v.guildNote);
		end,
	},
	officernote = {
		Label = L["OfficerNote"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_OFFICER, v.officerNote);
		end,
		CanUse = function() return CanViewOfficerNote() end,
	},
	notecombi = {
		Label = L["Note"],
		Brush = function(v)
			local normal = false;
			local officer = false;
			
			if( v.guildNote and v.guildNote ~= "" ) then
				normal = true;
			end
			if( CanViewOfficerNote() and v.officerNote and v.officerNote ~= "" ) then
				officer = true;
			end
			
			local note = NULL;
			
			if( normal and not officer ) then
				note = ("%s%s|r"):format(COLOR_GOLD, v.guildNote);
			elseif( officer and not normal ) then
				note = ("%s%s|r"):format(COLOR_OFFICER, v.officerNote);
			elseif( normal and officer ) then
				note = ("%s%s|r / %s%s|r"):format(COLOR_GOLD, v.guildNote, COLOR_OFFICER, v.officerNote);
			end
			
			return note;
		end,
	},
	acmpoints = {
		Label = L["Points"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v.acmPoints);
		end,
	},
};

------------------------------------------
-- GetColumns
------------------------------------------

function iGuild:GetColumns()
	local columns = self.db.Display;
	local new = {};

	for i = 1, #columns do
		local insert = true;
		local canUse = self.Columns[columns[i]].CanUse;
		
		if( type(canUse) == "boolean" and not canUse ) then
			insert = false;
		elseif( type(canUse) == "function" and not canUse() ) then
			insert = false;
		end
		
		if( insert ) then
			table.insert(new, columns[i]);
		end
	end
	
	return new;
end

------------------------------------------
-- OnInitialize
------------------------------------------

function iGuild:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("iGuildDB", self:CreateDB(), "Default").profile;
	
	self.GuildRoster = {};

	self:RegisterEvent("GUILD_MOTD", "EventHandler");
	self:RegisterEvent("PLAYER_GUILD_UPDATE", "EventHandler");
	
	if( IsInGuild() ) then
		self:RegisterEvent("GUILD_ROSTER_UPDATE", "EventHandler");
		GuildRoster();
		RosterTimer = self:ScheduleRepeatingTimer(GuildRoster, 15);
	else
		self.Broker.text = L["No guild"];
	end
end

------------------------------------------
-- EventHandler
------------------------------------------

function iGuild:EventHandler(event)
	if( IsInGuild() ) then
		if( not RosterTimer ) then
			self:RegisterEvent("GUILD_ROSTER_UPDATE", "EventHandler");
			GuildRoster();
			RosterTimer = self:ScheduleRepeatingTimer(GuildRoster, 15);
		end
		
		local Total, TotalOnline = GetNumGuildMembers();
		self.Broker.text = ("%d/%d"):format(TotalOnline, Total);
		
		self:SetupGuildRoster();
	else
		if( RosterTimer ) then
			self:UnregisterEvent("GUILD_ROSTER_UPDATE");
			self:CancelTimer(RosterTimer);
		end
		
		self.GuildRoster = {};
		self.Broker.text = L["No guild"];
	end
	
	if( QTip:IsAcquired(self) ) then
		self:UpdateTooltip();
	end
end

------------------------------------------
-- SetupGuildRoster
------------------------------------------

function iGuild:SetupGuildRoster()
	local roster = {};
	local _, TotalOnline = GetNumGuildMembers();
	
	for i = 1, TotalOnline do
		local charName, guildRank, _, charLevel, charClass, charZone, 
			guildNote, officerNote, isOnline, charStatus, _, acmPoints, acmRank, charMobile = GetGuildRosterInfo(i);
		
		if( isOnline ) then
			local vartable = {
				charName = charName,
				charLevel = charLevel,
				charClass = charClass,
				charZone = charZone,
				charStatus = charStatus or nil,
				charMobile = charMobile or false,
				acmPoints = acmPoints,
				acmRank = acmRank,
				guildRank = guildRank,
				guildNote = guildNote or nil,
				officerNote = officerNote or nil,
			};
			
			roster[i] = vartable;
		end
	end
	
	table.sort(roster, self.Sort[self.db.Sort]);
	self.GuildRoster = roster;
end

------------------------------------------
-- OnClick callbacks
------------------------------------------

local function cbTooltipLineClick(_, name, button)
	if( IsAltKeyDown() ) then
		InviteUnit(name);
	else
		SetItemRef(("player:%s"):format(name), ("|Hplayer:%s|h[%s]|h"):format(name, name), "LeftButton");
	end
end

------------------------------------------
-- UpdateTooltip
------------------------------------------

function iGuild:UpdateTooltip()
	Tooltip:Clear();
	
	local ActiveCols = self:GetColumns();
	Tooltip:SetColumnLayout(#ActiveCols);
	
	for NumLine = COLUMN_HEADER_ROW, #self.GuildRoster do
		local member;

		for NumCol = 1, #ActiveCols do
			local ColName = ActiveCols[NumCol];
			local ColInfo = self.Columns[ColName];
			
			if( NumCol == 1 ) then
				if( NumLine == COLUMN_HEADER_ROW) then
					Tooltip:AddHeader(" ");
				else
					Tooltip:AddLine(" ");
				end
			end
			
			if( NumLine == COLUMN_HEADER_ROW ) then
				local label = "";
				if( self.db.Column[ColName].ShowLabel ) then
					label = ColInfo.Label;
				end
				
				Tooltip:SetCell(NumLine +1, NumCol, label, nil, self.db.Column[ColName].Align);
			else
				member = self.GuildRoster[NumLine];
				
				Tooltip:SetCell(NumLine +1, NumCol, ColInfo.Brush(member), nil, self.db.Column[ColName].Align);
				if( ColInfo.Script and ColInfo.ScriptUse() ) then
					Tooltip:SetCellScript(NumLine +1, NumCol, "OnMouseDown", ColInfo.Script, member);
				end
				
			end
		end
		
		if( member ) then
			Tooltip:SetLineScript(NumLine +1, "OnMouseDown", cbTooltipLineClick, member.charName);
		end
	end
end